# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
https://core.ac.uk/

Aggregating the world's open access research papers.

We offer seamless access to millions of open access research papers, enrich the
collected data for text-mining and provide unique services to the research
community.

CORE aggregates research papers from data providers from all over the world
including institutional repositories, subject-repositories and journal
publishers. This process, which is called harvesting, allows us to offer search,
text-mining and analytical capabilities over not only metadata, but also the
full-text of the research papers making CORE a unique service in the research
community.

CORE currently contains 83,326,016 open access articles, from over tens of
thousands journals, collected from over 3,667 repositories around the world. 

----

* https://scholarlycommunications.jiscinvolve.org/wp/2017/12/14/enhancing-library-discovery-services-with-core-content/

> The CORE service is working in partnership with ProQuest to deliver more
> content within their library discovery services (Ex Libris Primo and Ex Libris
> Summon).
"""

from __future__ import print_function

import datetime

import luigi
from gluish.utils import shellout
from gluish.parameter import ClosestDateParameter

from siskin.task import DefaultTask


class CoreTask(DefaultTask):
    """
    Base task. Irregular updates.
    """
    TAG = 'core'

    def closest(self):
        return datetime.date(2017, 11, 1)


class CoreDownload(CoreTask):
    """
    Metadata. Filenaming inconsistent (tar or tar.gz).
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        output = shellout("""curl --fail https://core.ac.uk/datasets/core_{date}_metadata.tar > {output}""",
                          date=self.closest().strftime("%Y-%m-%d"))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="tar.gz"))

class CoreDownloadFulltext(CoreTask):
    """
    Metadata and Fulltext. Filenaming inconsistent (tar or tar.gz).
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        output = shellout("""curl --fail https://core.ac.uk/datasets/core_{date}_fulltext.tar > {output}""",
                          date=self.closest().strftime("%Y-%m-%d"))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="tar.gz"))




#!/usr/bin/env python
# coding: utf-8

from builtins import *
import io
import re
import sys

import pymarc
import marcx

copytags = ("002", "003", "004", "005", "006", "007", "008", "009", "010", "011", "012", "013", "014",
            "016", "017", "018", "022", "024", "030", "035", "040", "041", "100", "110", "111",
            "240", "242", "243", "245", "246", "247", "249", "250", "260", "263", "300", "310", "362",
            "490", "500", "501", "502", "504", "505", "510", "515", "516", "538", "546", "547", "550",
            "590", "600", "610", "611", "630", "648", "649", "651", "655", "700", "710", "711", "730",
            "770", "772", "773", "775", "776", "780", "785", "787", "800", "810", "811", "830", "856",
            "906", "982", "999")

# Default input and output.
inputfilename, outputfilename = "148_input.mrc", "148_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = io.open(inputfilename, "rb")
outputfile = io.open(outputfilename, "wb")

reader = pymarc.MARCReader(inputfile)

for oldrecord in reader:

    newrecord = marcx.Record(force_utf8=True)

    # prüfen, ob ID vorhanden ist
    if not oldrecord["001"]:      
        continue

    # prüfen ob Titel vorhanden ist
    if not (oldrecord["245"] and oldrecord["245"]["a"]):    
        continue

    # leader
    leader = "     " + oldrecord.leader[5:]
    newrecord.leader = leader

    # 001
    f001 = oldrecord["001"].data
    newrecord.add("001", data="finc-148-%s" % f001)

    # ISBN
    try:
        f020a = oldrecord["020"]["a"]        
    except:
        f020a = ""

    if f020a != "":
        f020a = f020a.replace(" ", "-")
        f020a = f020a.replace(".", "-")       
        regexp = re.search("([0-9xX-]{10,17})", f020a)
     
        if regexp:
            f020a = regexp.group(1)
            f020a = f020a.rstrip("-")
            newrecord.add("020", a=f020a) 
        else:
            print("Die ISBN %s konnte nicht mittels regulärer Ausdrücke überprüft werden." % f020a)

    # Originalfelder, die ohne Änderung übernommen werden
    for tag in copytags:
        for field in oldrecord.get_fields(tag):
            newrecord.add_field(field)

    # Schlagwort
    try:
        f689a = oldrecord["650"]["a"]     
        newrecord.add("689", a=f689a)
    except (AttributeError, TypeError):
        pass

    #n Link zum Datensatz
    newrecord.add("856", q="text/html", _3="Link zum Bundesarchiv", u="https://apps.bundesarchiv.de/F?func=find-c&ccl_term=SYS%3D" + f001 + "&local_base=BAB01")

    # 980
    collections = ["a", f001, "b", "148", "c", "Bundesarchiv (Filmarchiv)", "c", "Verbundkatalog Film"]
    newrecord.add("980", subfields=collections)
  
    outputfile.write(newrecord.as_marc())

inputfile.close()
outputfile.close()


# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2016 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Gutenberg
---------

https://www.gutenberg.org/, refs #10875, #5520.

"""

import datetime

import luigi
from gluish.intervals import weekly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout

from siskin.task import DefaultTask


class GutenbergTask(DefaultTask):
    """
    Base task for Gutenberg.
    """
    TAG = '1'

    def closest(self):
        return weekly(date=self.date)


class GutenbergDownload(GutenbergTask):
    """
    Download RDF dump.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        output = shellout("wget -O {output} http://www.gutenberg.org/cache/epub/feeds/rdf-files.tar.zip")
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='tar.zip'))


class GutenbergMARC(GutenbergTask):
    """
    Extract and convert.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return GutenbergDownload(date=self.date)

    def run(self):
        output = shellout("unzip -p {input} | tar -xO | python {script} - > {output}",
                          input=self.input().path, script=self.assets('1/1_marcbinary.py'))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
IMSLP, refs #1240.

Config
------

[imslp]

url = http://example.com/imslpOut_2016-12-25.tar.gz
"""

import datetime
import shutil
import tempfile

import luigi
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout

from siskin.task import DefaultTask


class IMSLPTask(DefaultTask):
    """ Base task for IMSLP. """
    TAG = "15"

    def closest(self):
        return datetime.date(2017, 12, 25)


class IMSLPDownload(IMSLPTask):
    """ Download raw data. Should be a single URL pointing to a tar.gz. """

    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        output = shellout("""wget -O {output} {url}""", url=self.config.get('imslp', 'url'))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='tar.gz'))


class IMSLPConvert(IMSLPTask):
    """
    Extract and transform.

    TODO, refs #13055.

    File "/usr/lib/python2.7/site-packages/siskin/assets/15/15_marcbinary.py", line 165, in <module>
        record = record["document"]
    KeyError: 'document'
    """

    date = ClosestDateParameter(default=datetime.date.today())
    debug = luigi.BoolParameter(description='do not delete temporary folder')

    def requires(self):
        return IMSLPDownload(date=self.date)

    def run(self):
        tempdir = tempfile.mkdtemp(prefix='siskin-')
        shellout("tar -xzf {archive} -C {tempdir}", archive=self.input().path, tempdir=tempdir)
        output = shellout("python {script} {tempdir} {output}",
                          script=self.assets('15/15_marcbinary.py'), tempdir=tempdir)
        if not self.debug:
            shutil.rmtree(tempdir)
        else:
            self.logger.debug("not deleting temporary folder at %s", tempdir)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))



#!/usr/bin/env python
# coding: utf-8

from builtins import *

import io
import sys
import re
import base64

import marcx
import xmltodict

lang_map = {
    "deu": "ger"
}

# Default input and output.
inputfilename = "73_input.xml" 
outputfilename = "73_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = io.open(inputfilename, "rb")
outputfile = io.open(outputfilename, "wb")
xmlfile = inputfile.read()
xmlrecords = xmltodict.parse(xmlfile)

for xmlrecord in xmlrecords["Records"]["Record"]:
    
    marcrecord = marcx.Record(force_utf8=True)

    # Leader
    marcrecord.leader = "     nab  22        4500"

    # Identifier
    f001 = xmlrecord["header"]["identifier"]
    f001 = f001.encode("utf-8")
    f001 = base64.b64encode(f001)
    f001 = f001.decode("ascii")
    marcrecord.add("001", data="finc-73-" + f001)

    # 007
    marcrecord.add("007", data="cr")

    # ISSN
    try:
        f022 = xmlrecord.get("metadata").get("oai_dc:dc").get("dc:source", "")
        f022a = f022[1]
        # elektronisch
        marcrecord.add("022", a=f022a)
        # print
        f022a = f022[2]
        marcrecord.add("022", a=f022a)
    except:
        pass

    # Sprache
    try:
        language = xmlrecord.get("metadata").get("oai_dc:dc").get("dc:language", "")
        f041a = lang_map.get(language, "")
        if f041a != "":
            marcrecord.add("008", data="130227uu20uuuuuuxx uuup%s  c" % f041a)
            marcrecord.add("041", a=f041a)          
        else:
            print("Die Sprache %s fehlt in der Lang_Map!" % language)
    except:
        pass

    # 1. Urheber
    try:
        f100a = xmlrecord.get("metadata").get("oai_dc:dc").get("dc:creator", "")
        marcrecord.add("100", a=f100a)
    except:
        pass

    # Haupttitel, Titelzusatz, Verantwortlichenangabe
    try:
        f245 = xmlrecord.get("metadata").get("oai_dc:dc").get("dc:title", "").get("#text")
        f245 = f245.split(":")
        f245a = f245[0] 
        f245a = f245a.strip(" ")
        if len(f245) > 1:
            f245a += " – " + f245[1].strip(" ")
        if len(f245) > 2: 
            f245b = ""       
            for titlepart in f245[2:]:
                titlepart = titlepart.strip(" ")
                f245b += titlepart + " : "
        else:
            f245b = ""        
        marcrecord.add("245", a=f245a, b=f245b.rstrip(" : "))
    except:
        continue

    # Erscheinungsvermerk
    f260c = xmlrecord.get("metadata").get("oai_dc:dc").get("dc:date", "")
    regexp = re.search("(\d\d\d\d)", f260c)
    if regexp:
        f260c = regexp.group(1)
    publisher = ["a", "Marburg", "b", " : " + "Schüren Verlag, ", "c", f260c]
    marcrecord.add("260", subfields=publisher)

    # Schlagwort
    try:
        f689a = xmlrecord.get("metadata").get("oai_dc:dc").get("dc:subject", "").get("#text")
        marcrecord.add("689", a=f689a)
    except:
        pass

    # übergeordnete Ressource
    f773 = xmlrecord.get("metadata").get("oai_dc:dc").get("dc:source", "")
    f773 = f773[0]
    f773g = f773.get("#text")

    regexp1 = re.search("Nr\.\s(\d.*?)\s\((\d\d\d\d)\);\s(\d+)", f773g)
    regexp2 = re.search(";\s(\d\d\d\d):\s(.*?);\s(\d+)", f773g)

    if regexp1:  
        issue = regexp1.group(1)
        year =  regexp1.group(2)
        startpage= regexp1.group(3)
        f773g = "(%s) Heft %s, S. %s-" % (year, issue, startpage)    
        marcrecord.add("773", g=f773g, t="MEDIENwissenschaft: Rezensionen | Reviews")
    elif regexp2:       
        year =  regexp2.group(1)
        issue = regexp2.group(2)
        startpage= regexp2.group(3)
        f773g = "(%s), %s, S. %s-" % (year, issue, startpage)    
        marcrecord.add("773", g=f773g, t="MEDIENwissenschaft: Rezensionen | Reviews")       
    else:
        print("Regexp konnte nicht gelesen werden: %s" % f773)
    
    # Link zur Ressource
    f856u = xmlrecord.get("metadata").get("oai_dc:dc").get("dc:relation", "")
    marcrecord.add("856", q="text/html", _3="Link zur Ressource", u=f856u)

    # Medienform
    marcrecord.add("935", b="cofz")

    # Kollektion
    # marcrecord.add("980", a=f001, b="73", c="Medienwissenschaft, Rezensionen, Reviews")
    marcrecord.add("980", a=f001, b="73", c="MedienwRezensionen")

    
    outputfile.write(marcrecord.as_marc())


inputfile.close()
outputfile.close()


# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301
# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
JOVE refs #11308.
"""

import csv
import datetime
import io
import json
import os
import sys

import luigi
import requests
from gluish.utils import shellout

from siskin.decorator import deprecated
from siskin.task import DefaultTask


class JoveTask(DefaultTask):
    TAG = "143"


class JoveMARC(JoveTask):
    """
    Turn MRK MARC (https://www.loc.gov/marc/makrbrkr.html) into MARC XML via
    experimental marctexttoxml (https://git.io/vdCUN).
    """

    debug = luigi.BoolParameter(default=False, significant=False)
    date = luigi.DateParameter(default=datetime.date.today())

    def run(self):
        cleanup = []

        url = 'https://www.jove.com/api/articles/v0/articlefeed.php?marc=1&sections[]=0&sections[]=4'
        output = shellout(
            """curl -sLg '{url}' | head -n -1 > {output}""", url=url)
        cleanup.append(output)

        output = shellout(
            "marctexttoxml < {input} | xmllint --format - > {output}", input=output)
        luigi.LocalTarget(output).move(self.output().path)

        for item in cleanup:
            try:
                os.remove(item)
            except OSError:
                self.logger.warn('could not delete temporary files')

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))


class JoveFincMARC(JoveTask):
    """
    Convert JOVE to fincmarc.
    """
    date = luigi.DateParameter(default=datetime.date.today())

    def requires(self):
        return JoveMARC(date=self.date)

    def run(self):
        output = shellout("""python {script} {input} {output}""",
                          script=self.assets("143/143_marcbinary.py"),
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))


class JoveIntermediateSchemaFromCSV(JoveTask):
    """
    Download and convert. Just a prototype working with the CSV output.
    """

    @deprecated
    def run(self):
        r = requests.get(
            "https://www.jove.com/api/articles/v0/articlefeed.php?csv=1")
        s = io.StringIO(r.text)
        with self.output().open('w') as output:
            for doc in csv.DictReader(s):
                converted = {
                    'doi': doc['DOI'],
                    'url': [doc['DOI Link'], doc['Link']],
                    'rft.atitle': doc['Article Title'],
                    'rft.jtitle': doc['Journal'],
                    'authors': [{"rft.au": name} for name in doc.get('Authors').split(';')],
                    'finc.record_id': doc['ArticleID'],
                    'finc.source_id': 'jove',
                    'finc.id': 'ai-jove-%s' % doc['ArticleID'],
                    'x.subjects': doc['Section'].split(','),
                    'rft.date': datetime.datetime.strptime(doc['Date Published'], '%m/%d/%Y').strftime("%Y-%m-%d"),
                    'rft.genre': 'article',
                    'version': '0.9',
                    'ris.type': 'EJOUR',
                    'finc.mega_collection': 'JoVE',
                    'abstract': doc.get('Summary'),
                }
                output.write(json.dumps(converted))
                output.write("\n")

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))



# coding: utf-8
#
# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Filmmusikforschung (FMF), refs #7967.

Note: These tasks are based on an XML version of the original XLSX, you need to
configure that.

Configuration:

[kielfmf]

input = /path/to/xml (ask RS)
"""

import luigi
from gluish.utils import shellout
from luigi.format import Gzip

from siskin.task import DefaultTask


class KielFMFTask(DefaultTask):
    """
    This task inherits functionality from `siskin.task.DefaultTask`.
    """
    TAG = '101'


class KielFMFIntermediateSchema(KielFMFTask):
    """
    Create intermediate schema.
    """

    def run(self):
        # We need an XML file. Uee a config entry to point to that file.
        input = self.config.get('kielfmf', 'input')

        # shellout run shell command and allows placeholders. The {output}
        # placeholder is a bit special. If it is not set explicitly, it will
        # point to a temporary file. The return value of shellout is the value of
        # output.
        output = shellout("""
            flux.sh {flux} inputfile={input} | pigz -c > {output}
        """, flux=self.assets('101/101_flux.flux'), input=input)

        # This moves the output of the above shell command into the desired task
        # output path.
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'), format=Gzip)



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
archive.org, refs #8000
"""

import datetime

import luigi
from gluish.common import Executable
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout

from siskin.task import DefaultTask


class ArchiveTask(DefaultTask):
    TAG = "153"

    def closest(self):
        return monthly(date=self.date)


class ArchiveDownload(ArchiveTask):
    """
    OAI harvest.
    """

    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        url = "http://archive.org/services/oai.php"
        collection = "collection:prelinger"
        shellout("""metha-sync -set {collection} {url}""",
                 collection=collection, url=url)
        output = shellout("""metha-cat -set {collection} {url} > {output}""",
                          url=url, collection=collection)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="xml"))


class ArchiveMARC(ArchiveTask):
    """
    Convert.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return ArchiveDownload(date=self.date)

    def run(self):
        output = shellout("python {script} {input} {output}",
                          script=self.assets("153/153_marcbinary.py"),
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))


class ArchiveSearch(ArchiveTask):
    """
    Search archive via the ia tool. Requires Archive.org account:

    https://archive.org/account/login.createaccount.php

    The command `ia configure` will set you up.

        $ cat ~/.config/ia.ini
        [s3]
        access = asudiasd77xsdlds
        secret = oasdu888s8x9a0sd

        [cookies]
        logged-in-user = user@example.com
        logged-in-sig = secret

    Refs #8000.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    query = luigi.Parameter(default='collection:prelinger')

    def requires(self):
        """
        The setup of siskin should install this automatically.
        """
        return Executable(name='ia',
                          message='https://pypi.python.org/pypi/internetarchive')

    def run(self):
        output = shellout("ia search '{query}' > {output}", query=self.query)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj', digest=True))


class ArchiveSearchMetadata(ArchiveTask):
    """
    For a given search query, harvest all metadata. This can take a long time,
    since each record is requested separately. Maybe there is some faster way.

    Currently 1 record/s.

    Refs #8000.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    query = luigi.Parameter(default='collection:prelinger')

    def requires(self):
        return ArchiveSearch(date=self.date, query=self.query)

    def run(self):
        output = shellout("for i in $(jq -r .identifier {input}); do ia metadata $i; done >> {output}",
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj', digest=True))

class ArchiveTelevisionTexts(ArchiveTask, luigi.WrapperTask):
    """
    Temporary task for harvesting special query from archive, refs #8000.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        """
        loans__status__status:NULL (as in web) would not work.
        """
        query = 'subject:television AND mediatype:texts AND -loans__status__status:"AVAILABLE" AND -loans__status__status:"UNAVAILABLE"'
        return ArchiveSearchMetadata(date=self.date, query=query)

    def run(self):
        luigi.LocalTarget(self.input().path).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj', digest=True))



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2018 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Disson, Dissertations Online, refs #3422.
"""

import collections
import datetime
import json
import re

import luigi
import pymarc
import marcx
import tqdm

from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class DissonTask(DefaultTask):
    """
    Base task for disson.
    """
    TAG = '13'

    def closest(self):
        return monthly(date=self.date)


class DissonHarvest(DissonTask):
    """
    Via OAI, access might be restricted.

    ;; ANSWER SECTION:
    services.dnb.de.	3539	IN	CNAME	httpd-e02.dnb.de.
    httpd-e02.dnb.de.	3539	IN	CNAME	prodche-02.dnb.de.
    prodche-02.dnb.de.	3539	IN	A	    193.175.100.222

    Format options (2018-03-02):

    * baseDc-xml
    * BibframeRDFxml
    * JATS-xml
    * MARC21plus-1-xml
    * MARC21plus-xml
    * MARC21-xml
    * mods-xml
    * oai_dc
    * ONIX-xml
    * PicaPlus-xml
    * RDFxml
    * sync-repo-xml
    * xMetaDiss

    """
    date = ClosestDateParameter(default=datetime.date.today())
    format = luigi.Parameter(default='MARC21-xml')
    set = luigi.Parameter(default='dnb-all:online:dissertations')

    def run(self):
        endpoint = "http://services.dnb.de/oai/repository"
        shellout("metha-sync -daily -format {format} -set {set} {endpoint}",
                 format=self.format, set=self.set, endpoint=endpoint)
        output = shellout("""metha-cat -format {format} -set {set} {endpoint} |
                             pigz -c > {output}""", format=self.format, set=self.set,
                          endpoint=endpoint)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml.gz', digest=True))


class DissonMARC(DissonTask):
    """
    Create a binary MARC version.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return DissonHarvest(date=self.date)

    def run(self):
        output = shellout("yaz-marcdump -i marcxml -o marc <(unpigz -c {input}) > {output}",
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))


class DissonIntermediateSchema(DissonTask):
    """
    Convert MARC to intermediate schema.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return DissonMARC(date=self.date)

    def run(self):

        stats = collections.Counter()

        def parse_date(record):
            """
            XXX: Factor this out.
            """
            v = rr.firstvalue('264.c')
            if not v:
                v = rr.firstvalue('502.a')
                if not v:
                    v = rr["005"].value()[:4]
                    if not v:
                        raise RuntimeError('unparsed date (no 264.a, 502.a, 005):\n%s' % record)
                    stats['005'] += 1
                else:
                    stats['502.a'] += 1

                match = re.search(r'.*(\d\d\d\d).*', v)
                if match:
                    v = match.group(1)
                else:
                    raise RuntimeError("ignoring broken (%s) record: %s" % (v, rr["001"].value()))

            else:
                stats['264.c'] += 1

            v = re.sub("[^0-9]", "", v)
            if v and len(v) == 4:
                return '%s-01-01' % v

            raise RuntimeError('unparsed date %s: %s' % (v, record))

        with self.input().open() as handle:
            with self.output().open('w') as output:
                reader = pymarc.MARCReader(handle, to_unicode=True)
                for record in tqdm.tqdm(reader):
                    rr = marcx.Record().from_record(record)
                    identifier = rr['001'].value()

                    try:
                        date = parse_date(rr)
                    except RuntimeError as err:
                        self.logger.debug(u"%s" % err)
                        stats['ignored'] += 1
                        continue

                    doc = {
                        'finc.source_id': '13',
                        'finc.record_id': identifier,
                        'finc.id': 'ai-13-%s' % (identifier),
                        'rft.date': date,
                        'urls': list(rr.itervalues('856.u')),
                        'rft.btitle': rr.firstvalue('245.a'),
                        'subjects': list(rr.itervalues('689.a', '650.a')),
                        'authors': [{'rft.au': name} for name in rr.itervalues('100.a')],
                    }
                    output.write("%s\n" % json.dumps(doc))

        self.logger.debug(json.dumps(stats.most_common()))

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
SSOAR workflow with OAI harvest and metafacture.
"""

import datetime

import luigi
from gluish.format import Gzip
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout

from siskin.task import DefaultTask


class SSOARTask(DefaultTask):
    """ Base task for SSOAR """
    TAG = '30'

    def closest(self):
        return monthly(date=self.date)


class SSOARHarvest(SSOARTask):
    """
    Harvest from default http://www.ssoar.info/OAIHandler/request OAI endpoint.
    """
    format = luigi.Parameter(default="marcxml", significant=False)
    endpoint = luigi.Parameter(
        default='http://www.ssoar.info/OAIHandler/request', significant=False)
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        shellout("""metha-sync -format {format} {endpoint} """, endpoint=self.endpoint,
                 format=self.format)
        output = shellout("""metha-cat -format {format} -root Records {endpoint} > {output}""", endpoint=self.endpoint,
                          format=self.format)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))


class SSOARIntermediateSchema(SSOARTask):
    """
    Convert to intermediate schema via metafacture. Custom morphs and flux are
    kept in assets/30. Maps are kept in assets/maps.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return SSOARHarvest(date=self.date)

    def run(self):
        mapdir = 'file:///%s' % self.assets("maps/")
        output = shellout("""flux.sh {flux} in={input} MAP_DIR={mapdir} | pigz -c > {output}""",
                          flux=self.assets("30/30.flux"), mapdir=mapdir, input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'), format=Gzip)


class SSOARExport(SSOARTask):
    """
    Export to finc solr schema by using span-export.
    """
    format = luigi.Parameter(default='solr5vu3', description='export format')
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return SSOARIntermediateSchema(date=self.date)

    def run(self):
        output = shellout("""unpigz -c {input} | span-export -with-fullrecord -o {format} | pigz -c > {output}""",
                          format=self.format, input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincsolr.ndj.gz'), format=Gzip)
